/**
 * Copyright (c) 2020, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * WSO2 Inc. licenses this file to you under the Apache License,
 * Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
import { Container, Responsive, Sidebar } from "semantic-ui-react";
import React, { useEffect, useState } from "react";
import _ from "lodash";
import classNames from "classnames";
import { SidePanelItems } from "./side-panel-items";
import { UIConstants } from "@wso2is/core/constants";
/**
 * Side panel base component.
 *
 * @param {SidePanelPropsInterface} props - Props injected to the component.
 *
 * @return {React.ReactElement}
 */
export const SidePanel = (props) => {
    const { bordered, children, desktopContentTopSpacing, footerHeight, fluid, headerHeight, mobileSidePanelVisibility, onSidePanelItemClick, onSidePanelPusherClick, routes } = props;
    const [items, setItems] = useState(routes);
    const wrapperClasses = classNames("side-panel-wrapper", {
        bordered,
        [typeof bordered === "boolean" ? "all" : `${bordered}`]: bordered
    });
    const mainLayoutStyles = {
        paddingBottom: `${footerHeight}px`,
        paddingTop: `${headerHeight}px`
    };
    const desktopContentStyle = {
        minHeight: `calc(100vh - ${headerHeight + footerHeight}px`,
        paddingTop: `${desktopContentTopSpacing}px`
    };
    /**
     * Update items when the routes prop changes
     */
    useEffect(() => {
        setItems(routes);
    }, [routes]);
    /**
     * Evaluate if the child item section should be extended or not. If so, adds
     * `open` attribute to the route section.
     *
     * @param {RouteInterface[] | ChildRouteInterface[]} routesArray - Array of routes.
     * @param {RouteInterface | ChildRouteInterface} route - Evaluating route.
     *
     * @return {RouteInterface[]} Modified set of routes.
     */
    const evaluateSidePanelItemExtension = (routesArray, route) => {
        return _.filter([...routesArray], (evalRoute) => {
            if (evalRoute.id === route.id) {
                evalRoute.open = !evalRoute.open;
            }
            if (evalRoute.children) {
                evaluateSidePanelItemExtension(evalRoute.children, route);
            }
            return evalRoute;
        });
    };
    /**
     * Handles side panel item onclick.
     *
     * @param {RouteInterface | ChildRouteInterface} route - Clicked on route.
     */
    const handleItemOnClick = (route) => {
        setItems(evaluateSidePanelItemExtension(routes, route));
        onSidePanelItemClick(route);
    };
    return (React.createElement("div", { style: mainLayoutStyles, className: "layout-content" },
        React.createElement(Responsive, Object.assign({}, Responsive.onlyMobile, { className: "mobile-container" }),
            React.createElement(Sidebar.Pushable, null,
                React.createElement(Sidebar, { animation: "push", visible: mobileSidePanelVisibility },
                    React.createElement(SidePanelItems, Object.assign({}, props, { type: "mobile", onSidePanelItemClick: handleItemOnClick, routes: items }))),
                React.createElement(Sidebar.Pusher, { onClick: onSidePanelPusherClick, className: "side-panel-pusher" },
                    React.createElement("div", { className: "content-wrapper" }, children)))),
        React.createElement(Responsive, { as: Container, className: "desktop-container", fluid: fluid, style: desktopContentStyle, minWidth: Responsive.onlyTablet.minWidth },
            React.createElement("div", { className: wrapperClasses },
                React.createElement(SidePanelItems, Object.assign({}, props, { type: "desktop", onSidePanelItemClick: handleItemOnClick, routes: items }))),
            React.createElement("div", { className: "content-wrapper" }, children))));
};
/**
 * Default props for the side panel items component.
 */
SidePanel.defaultProps = {
    desktopContentTopSpacing: UIConstants.DEFAULT_DASHBOARD_LAYOUT_DESKTOP_CONTENT_TOP_SPACING,
    fluid: false,
    sidePanelItemHeight: UIConstants.DEFAULT_SIDE_PANEL_ITEM_HEIGHT,
    sidePanelTopMargin: false,
    translationHook: null
};
//# sourceMappingURL=side-panel.js.map